'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _es6GlobalParser = require('es6-global-parser');

var _es6GlobalParser2 = _interopRequireDefault(_es6GlobalParser);

var _escapeStringRegexp = require('escape-string-regexp');

var _escapeStringRegexp2 = _interopRequireDefault(_escapeStringRegexp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ReplaceAt = (original, replacement, start, end) => {
    return original.slice(0, start) + replacement + original.slice(end);
};

const SourceModifier = (source, filename, key, system) => {
    // console.log('Second!')
    const shortFileName = '.' + filename.replace(process.cwd(), '');
    const invocations = [];
    const substitutions = [];
    const suffixID = '__NUTRA_MOCK__';
    const originalsource = source;

    // Get all the global names that would need to be mocked
    const globals = (0, _es6GlobalParser2.default)(source);

    // Go through all the names to in reverse as to not distort the positions of
    // the static replacements
    globals.reverse().forEach(_ref => {
        let type = _ref.type;
        let name = _ref.name;
        let start = _ref.start;
        let end = _ref.end;

        // Add suffix to all global names to make it easier to do a global
        // replacement of all relevant names in the deep execution contexts
        source = ReplaceAt(source, name + suffixID, start, end);
        // While we are at it
        if (type !== 'export') {
            // Create all the invocations that will set the original values for
            // all the global names in the file
            invocations.push(`NutraMock.setEntry("${ shortFileName }", "${ name }", ${ name })`);
            // Create all the substitutions that will deliver the correct values
            // whenever these global names are requested
            substitutions.push({
                name,
                alias: `NutraMock.store["${ shortFileName }"]["${ name }"].fake`
            });
        }
    });

    // Replace all deep execution contexts names with their substitutes
    substitutions.forEach(_ref2 => {
        let name = _ref2.name;
        let alias = _ref2.alias;

        const escapedRegex = (0, _escapeStringRegexp2.default)(name);
        const pattern = new RegExp(`\\b${ name }\\b`, 'g');
        source = source.replace(pattern, alias);
    });

    // Restore all global names to their original value
    const pattern = new RegExp(`${ suffixID }`, 'g');
    source = source.replace(pattern, '');

    // Inject the invocations that will set the original values
    source = source + ';\n' + invocations.join(';\n');

    return source;
};

exports.default = SourceModifier;